// interface for mc6809.c
// by Mark Broadhead

#include "6809infc.h"
#include "../game/game.h"

extern game *g_game;	// our game class

static void FetchInstr(int addr, unsigned char fetch_buffer[])
{
    register int i;

    for (i=0; i<MC6809_FETCH_BUFFER_SIZE; i++)
	{
		fetch_buffer[i]=(g_game->cpu_mem_read(addr + i) & 0xff);
	}
}

static int LoadByte(int addr)
{
	return (g_game->cpu_mem_read(addr) & 0xff);
}

static int LoadWord(int addr)
{
	unsigned char high_byte = (g_game->cpu_mem_read(addr) & 0xff);
	unsigned char low_byte = (g_game->cpu_mem_read(addr + 1) & 0xff);
	return ((high_byte << 8) | low_byte);
}

static void StoreByte(int addr, int value)
{
	g_game->cpu_mem_write((addr & 0xffff), (value & 0xff));
}

static void StoreWord(int addr, int value)
{
	g_game->cpu_mem_write((addr & 0xffff), ((value >> 8) & 0xff));
	g_game->cpu_mem_write(((addr + 1) & 0xffff), (value & 0xff));
}

// I don't know if we'll need this...
static int BiosCall(struct MC6809_REGS *regs)
{
    return 0x12;  /* NOP */
}

void initialize_m6809(void)
{
    
	MC6809_INTERFACE interface={	FetchInstr,
									LoadByte,
									LoadWord,
									StoreByte,
									StoreWord,
									BiosCall };
    mc6809_Init(&interface);
	mc6809_Reset();
}


