
// mamewrap.c
// thrown together quickly by Matt Ownby

// the purpose of this code it to use cpu cores from MAME without modifying the CPU core source
// code (or at least with minimal modification)
// this file contains wrapper functions for functions that the mame cpu cores expect.

// at the time of this writing (Jan 6, 2001) there are enough wrappers for the z80 core to work.
// Additional code might be needed to add other mame cpu cores (such as the x86 one)

#ifdef WIN32
#pragma warning (disable:4100)	// disable the warning about unreferenced formal parameters (MSVC++)
#endif

#include <stdio.h>

#include "../daphne.h"
#ifdef USE_M80
#include "m80.h"
#else
#include "z80.h"
#endif
#include "mamewrap.h"
#include "../game/game.h"

extern game *g_game;	// our global game class

UINT8 *	OP_ROM = NULL;							/* opcode ROM base */
UINT8 *	OP_RAM = NULL;							/* opcode RAM base */

offs_t	memory_amask = 0xFFFF;					/* memory address mask */
int mame_debug = 1;	// whether our debugger is running

///////////////////////////////////////////////////

// IRQ callback that gets executed when the ISR is called.  It immediately clears the IRQ line so
// that normal program execution can continue when the ISR returns

#ifdef USE_M80
Uint32 generic_m80_irq_callback()
{
	m80_set_irq_line(CLEAR_LINE);
	return 0xFF;
}
#else
int generic_z80_irq_callback(int irqline)
{

	z80_set_irq_line(0, CLEAR_LINE);

    return 0xff;
}
#endif

Uint32 generic_m80_nmi_callback()
{
	m80_set_nmi_line(CLEAR_LINE);
	return 0;
}


// reads a byte from a memory location
UINT8 cpu_readmem16(UINT32 addr)
{

	return(g_game->cpu_mem_read( (Uint16) addr));

}

// writes a byte to a memory location
void cpu_writemem16(UINT32 addr, UINT8 value)
{

	g_game->cpu_mem_write( (Uint16) addr, value);

}

// reads data from a port
UINT8 cpu_readport16(UINT16 port)
{

	return(g_game->port_read(port));

}

// outputs data to a port
void cpu_writeport16(UINT16 port, UINT8 value)
{

	g_game->port_write(port, value);

}

// returns the current active cpu.  Since we only support 1 CPU at this time, this function
// always returns a 0
unsigned char cpu_getactivecpu()
{

	return(0);

}

// notifies us that the PC has changed, but it doesn't notify us every time =(
void change_pc16(UINT32 new_pc)
{

	g_game->update_pc(new_pc);

}


///////////////////////////////////////////////////////////////////////////
// the rest of these functions are unsupported and do nothing


void logerror(const char *text, ...)
{

}

void state_save_UINT8(void *state, const char *module,int instance,
	const char *name, const UINT8 *val, unsigned size)
{
}

void state_save_INT8(void *state, const char *module,int instance,
	const char *name, const INT8 *val, unsigned size)
{
}

void state_save_UINT16(void *state, const char *module,int instance,
	const char *name, const UINT16 *val, unsigned size)
{
}

void state_save_INT16(void *state, const char *module,int instance,
	const char *name, const INT16 *val, unsigned size)
{
}

void state_load_UINT8(void *state, const char *module,int instance,
	const char *name, UINT8 *val, unsigned size)
{
}

void state_load_INT8(void *state, const char *module,int instance,
	const char *name, INT8 *val, unsigned size)
{
}

void state_load_UINT16(void *state, const char *module,int instance,
	const char *name, UINT16 *val, unsigned size)
{
}

void state_load_INT16(void *state, const char *module,int instance,
	const char *name, INT16 *val, unsigned size)
{
}
