/*
 * daphne.h
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// daphne.h

#ifndef DAPHNE_H
#define DAPHNE_H

// global definitions ...

#define LOGNAME "daphne_log.txt"	// name of our logfile if we are using one

//#define F2_LATENCY	150		// search latency that works well for rev F2 of Dragon's Lair
#define LED_RANGE 16

enum { GAME_UNDEFINED, GAME_LAIR, GAME_ACE, GAME_CLIFF, GAME_GTG, GAME_SUPERD, GAME_THAYERS, GAME_ASTRON, GAME_GALAXY, GAME_ESH, GAME_LAIREURO, GAME_BADLANDS }; // game types
enum { B_DL_PLAYER1, B_DL_PLAYER2, B_DL_LIVES, B_DL_CREDITS, B_DAPHNE_SAVEME, B_GAMENOWOOK, B_EMPTY }; // bitmaps
enum { FONT_SMALL, FONT_BIG };	// font enumeration, dependent on which order font .bmp's are loaded in
enum
{
	SWITCH_UP, SWITCH_LEFT, SWITCH_DOWN, SWITCH_RIGHT, SWITCH_BUTTON1, SWITCH_BUTTON2,
	SWITCH_BUTTON3, SWITCH_COIN1, SWITCH_COIN2, SWITCH_SKILL1, SWITCH_SKILL2, SWITCH_SKILL3,
	SWITCH_SERVICE, SWITCH_TEST, SWITCH_RESET, SWITCH_SCREENSHOT, SWITCH_QUIT, SWITCH_COUNT
}; // daphne inputs for arcade and additional controls, leave SWITCH_COUNT at the end

enum { MOD_NONE, MOD_SA91, MOD_PAL_DL, MOD_PAL_SA, MOD_PAL_DL_SC, MOD_PAL_SA_SC }; // frame modifiers
//enum { SKILL_CADET, SKILL_CAPTAIN, SKILL_ACE, SKILL_UNSPECIFIED }; // space ace skills
enum { CPU_UNDEFINED, CPU_Z80, CPU_X86, CPU_M6809 };	// cpu's supported by daphne now or in the future =]

void addlog(char *s);
void printerror(char *s);
int load_roms();
bool load_rom(char *, unsigned char *, unsigned int);
unsigned char get_filename(char *s, unsigned char n);
void set_quitflag();
unsigned char get_quitflag();
int main(int, char **);
void set_unix_signals();
void handle_unix_signals(int);
void set_serial_port(unsigned char i);
unsigned char get_serial_port();
void set_baud_rate(int i);
int get_baud_rate();
void set_vldp_ip(char *);
char *get_vldp_ip();
void set_vldp_port(int);
int get_vldp_port();
void set_search_offset(int i);
int get_search_offset();
void set_file_mask(char *mask);
char *get_file_mask();
unsigned char get_autostart();
void set_joystick_present();
unsigned char get_joystick_present();
void set_sound_enabled(unsigned char);
unsigned char get_sound_enabled();
unsigned char get_frame_modifier();
void set_frame_modifier(unsigned char value);
void set_scoreboard(unsigned char value);
unsigned char get_scoreboard();
void set_scoreboard_port(unsigned char value);
unsigned char get_scoreboard_port();
void delete_logfile();
unsigned int compute_checksum(unsigned char *buf, unsigned int buf_length);

//////////////////////////////////////////////////////////////////////////////////////

///////// temporary, remove after we have narrowed down to only 1 cpu

// comment this out to use the old z80 core
#define USE_M80	1

#ifdef USE_M80

#define Z80_GET_AF	m80_get_reg(M80_AF)
#define Z80_GET_HL	m80_get_reg(M80_HL)
#define Z80_GET_DE	m80_get_reg(M80_DE)
#define Z80_GET_BC	m80_get_reg(M80_BC)
#define Z80_GET_PC	m80_get_pc()
#define Z80_GET_SP	m80_get_sp()
#define Z80_GET_IX	m80_get_reg(M80_IX)
#define Z80_GET_IY	m80_get_reg(M80_IY)
#define Z80_GET_IFF1	0
#define Z80_SET_PC(val)	m80_set_pc(val)
#define Z80_SET_SP(val)	m80_set_sp(val)
#define Z80_SET_AF(val) m80_set_reg(M80_AF, val)
#define Z80_EXECUTE(val)	m80_execute(val)
#define Z80_ASSERT_IRQ	m80_set_irq_line(ASSERT_LINE)
#define Z80_CLEAR_IRQ	m80_set_irq_line(CLEAR_LINE)
#define Z80_ASSERT_NMI	m80_set_nmi_line(ASSERT_LINE); m80_execute(1); m80_set_nmi_line(CLEAR_LINE)
#define Z80_GET_ELAPSED_CYCLES	m80_get_cycles_executed()
#define Z80_EXIT
#define Z80_INCLUDE	"../cpu/m80.h"

#else
#define Z80_GET_AF	z80_get_reg(Z80_AF)
#define Z80_GET_HL	z80_get_reg(Z80_HL)
#define Z80_GET_DE	z80_get_reg(Z80_DE)
#define Z80_GET_BC	z80_get_reg(Z80_BC)
#define Z80_GET_PC	z80_get_pc()
#define Z80_GET_SP	z80_get_sp()
#define Z80_GET_IX	z80_get_reg(Z80_IX)
#define Z80_GET_IY	z80_get_reg(Z80_IY)
#define Z80_GET_IFF1	z80_get_reg(Z80_IFF1)
#define Z80_SET_PC(val)	z80_set_pc(val)
#define Z80_SET_SP(val)	z80_set_sp(val)
#define Z80_SET_AF(val) z80_set_reg(Z80_AF, val)
#define Z80_EXECUTE(val)	z80_execute(val)
#define Z80_ASSERT_IRQ	z80_set_irq_line(0, HOLD_LINE)
#define Z80_CLEAR_IRQ	z80_set_irq_line(0, CLEAR_LINE)
#define Z80_ASSERT_NMI	z80_set_nmi_line(ASSERT_LINE); z80_execute(1); z80_set_nmi_line(CLEAR_LINE)
#define Z80_GET_ELAPSED_CYCLES	printf("ERROR : Unsupported in Z80 core!\n");
#define Z80_EXIT		z80_exit();
#define Z80_INCLUDE	"../cpu/z80.h"
#endif


#endif
