/*
 * laireuro.cpp
 *
 * Copyright (C) 2001 Mark Broadhead
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// laireuro.cpp
// by Mark Broadhead
//
// this driver is very much incomplete, in fact, I shouldn't have submitted yet, hehe
//
// the main things it needs is emulation of the CTC Z8430A and DART Z8470 chips.
// in case anybody wants to work on this before I get back to it a datasheet that covers 
// those two chips can be found at http://www.zilog.com/docs/z80/um0081.pdf

#include <string.h>
#include "laireuro.h"
#include "../daphne.h"
#include Z80_INCLUDE
#include "../ldp-out/ldp.h"
#include "../io/conout.h"
#include "../video/video.h"

extern ldp *g_ldp;

laireuro::laireuro()
{
	memset(banks, 0xFF, 4);	// fill banks with 0xFF's
	memset(colors, 0, sizeof(colors));	// fill color palette with 0's
	video_overlay = SDL_CreateRGBSurface(SDL_HWSURFACE, LAIREURO_OVERLAY_W, LAIREURO_OVERLAY_H, 8, 0, 0, 0, 0); // create an 8-bit surface
	screen_needs_update = true;

	disc_fps = 25; // TODO: look up pal frame rate
	game_type = GAME_LAIREURO;
	cpu_type = CPU_Z80;
	cpu_hz = LAIREURO_CPU_HZ;
//	irq_period = LAIREURO_IRQ_PERIOD;
}


void laireuro::do_irq()
{
	// Redraws the screen (if needed) on interrupt
	display_update();
	Z80_ASSERT_IRQ;
}

Uint8 laireuro::cpu_mem_read(Uint16 addr)
{
	Uint8 result = cpumem[addr];

	return result;
}

void laireuro::cpu_mem_write(Uint16 Addr, Uint8 Value)
// Called whenever the Z80 emulator wants to write to memory
{	
	cpumem[Addr] = Value;

	// if the cpu writes to video memory, update the screen on the next interrupt
	if (Addr >= 0xc000 && Addr <= 0xc3ff)
	{
		screen_needs_update = true;
	}

}

void laireuro::port_write(Uint16 Port, Uint8 Value)
// Called whenever the emulator wants to output to a port
{

    char s[81] = { 0 };

    Port &= 0xFF;	// strip off high byte

    switch(Port)
	{
	

	default:
        sprintf(s, "LAIREURO: Unsupported Port Output-> %x:%x (PC is %x)", Port, Value, Z80_GET_PC);
        printline(s);
        break;

	}		
}

Uint8 laireuro::port_read(Uint16 Port)
// Called whenever the emulator wants to read from a port
{

    char s[81] = { 0 };
    unsigned char result = 0;

	Port &= 0xFF;	// strip off high byte

    switch(Port)
    {


    default:
        sprintf(s, "LAIREURO: Unsupported Port Input-> %x (PC is %x)", Port, Z80_GET_PC);
        printline(s);
        break;
    }

    return(result);

}

bool laireuro::load_roms()
{
	
	return (load_rom("laireuro/u45", &cpumem[0], 0x2000) &&
		load_rom("laireuro/u46", &cpumem[0x2000], 0x2000) &&
		load_rom("laireuro/u47", &cpumem[0x4000], 0x2000) &&
		load_rom("laireuro/u48", &cpumem[0x6000], 0x2000) &&

		load_rom("laireuro/u33", &character[0x0000], 0x2000)
		);
}

// initializes laireuro's video
bool laireuro::video_init()
{

	bool result = false;

	
	if (video_overlay)
	{
		SDL_SetColors(video_overlay, colors, 0, 32);
		result = true;
	}
	else
	{
		printline("ERROR: Could not create laireuro overlay");
	}

	return result;
}

// shuts down laireuro's video
void laireuro::video_shutdown()
{
	SDL_FreeSurface(video_overlay);
}

void laireuro::video_repaint()
{
	screen_needs_update = true;
	display_update();	
}

// updates laireuro's video
void laireuro::display_update()
{
	if (screen_needs_update)
	{
		SDL_Rect dest;	// setup place on screen to blit laireuro overlay
		dest.x = 0;	// upper left corner
		dest.y = 0;
		dest.w = (unsigned short) video_overlay->w;
		dest.h = (unsigned short) video_overlay->h;

		for (int charx = 0; charx < 32; charx++)
		{
			for (int chary = 0; chary < 32; chary++)
			{
				for (int x=0; x < 4; x++)
				{
					for (int y = 0; y < 8; y++)
					{
							Uint8 left_pixel = static_cast<Uint8>((character[cpumem[chary * 32 + charx + 0x5c00]*32+x+4*y] & 0xf0) >> 4);
							Uint8 right_pixel = static_cast<Uint8>((character[cpumem[chary * 32 + charx + 0x5c00]*32+x+4*y] & 0x0f));
							*((Uint8 *) video_overlay->pixels + ((chary * 8 + y) * LAIREURO_OVERLAY_W) + (charx * 8 + x * 2)) = left_pixel;
							*((Uint8 *) video_overlay->pixels + ((chary * 8 + y) * LAIREURO_OVERLAY_W) + (charx * 8 + x * 2 + 1)) = right_pixel;
					}
				}
			}
		}


//		SDL_BlitSurface(video_overlay, NULL, 

		dest.x = 0;	// upper left corner
		dest.y = 0;
		dest.w = (unsigned short) video_overlay->w;
		dest.h = (unsigned short) video_overlay->h;

		// SMPEG doesn't allow blitting, which is why we have to check
		if (g_ldp->is_blitting_allowed())
		{
			SDL_BlitSurface(video_overlay, NULL, get_screen(), &dest);
			SDL_Flip(get_screen());
		}
		screen_needs_update = false;
	}
	
	return;
}

// returns a pointer to Super Don's video overlay surface
// also returns width, height, and bits per pixel of surface
SDL_Surface *laireuro::get_video_overlay()
{
	return(video_overlay);
}

// returns which color in the palette is transparent.  In this case it's 0xF
Uint8 laireuro::get_transparent_color()
{
	return 0xF;
}

// returns a pointer to Super Don's video overlay color palette
SDL_Color *laireuro::get_video_colors()
{
	return colors;
}

// this gets called when the user presses a key or moves the joystick
void laireuro::input_enable(Uint8 move)
{
	switch (move)
	{
	case SWITCH_UP:
		banks[0] &= ~0x80; 
		break;
	case SWITCH_LEFT:
		banks[0] &= ~0x02;
		break;
	case SWITCH_RIGHT:
		banks[0] &= ~0x20;
		break;
	case SWITCH_DOWN:
		banks[0] &= ~0x08; 
		break;
	case SWITCH_BUTTON1: // '1' on keyboard
		banks[1] &= ~0x80; 
		break;
	case SWITCH_BUTTON2: // '2' on keyboard
		banks[1] &= ~0x40;
		break;
	case SWITCH_BUTTON3: // space on keyboard
		banks[1] &= ~0x20;
		break;
	case SWITCH_COIN1: 
		banks[1] &= ~0x08;
		break;
	case SWITCH_COIN2: 
		banks[1] &= ~0x04;
		break;
	case SWITCH_SERVICE: 
		banks[1] &= ~0x01;
		break;
	case SWITCH_TEST: 
		banks[1] &= ~0x02;
		break;
	default:
		printline("Error, bug in move enable");
		break;
	}
}  

// this gets called when the user releases a key or moves the joystick back to center position
void laireuro::input_disable(Uint8 move)
{
	switch (move)
	{
	case SWITCH_UP:
		banks[0] |= 0x80; 
		break;
	case SWITCH_LEFT:
		banks[0] |= 0x02;
		break;
	case SWITCH_RIGHT:
		banks[0] |= 0x20;
		break;
	case SWITCH_DOWN:
		banks[0] |= 0x08; 
		break;
	case SWITCH_BUTTON1: // '1' on keyboard
		banks[1] |= 0x80; 
		break;
	case SWITCH_BUTTON2: // '2' on keyboard
		banks[1] |= 0x40;
		break;
	case SWITCH_BUTTON3: // space on keyboard
		banks[1] |= 0x20;
		break;
	case SWITCH_COIN1: 
		banks[1] |= 0x08;
		break;
	case SWITCH_COIN2: 
		banks[1] |= 0x04;
		break;
	case SWITCH_SERVICE: 
		banks[1] |= 0x01;
		break;
	case SWITCH_TEST: 
		banks[1] |= 0x02;
		break;
	default:
		printline("Error, bug in move enable");
		break;
	}
}
