/*
 * seektest.cpp
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// seektest.cpp
// by Matt Ownby

// used to test mpeg seeking accuracy

#include <string.h>
#include "../io/conout.h"
#include "../ldp-out/ldp.h"
#include "seektest.h"
#include "../daphne.h"
#include "../io/input.h"

extern ldp *g_ldp;

seektest::seektest()
{
	disc_fps = 1.0;	// all these values ought to be changed.. use a preset
	strcpy(m_early1, "00001");
	strcpy(m_early2, "00001");
	strcpy(m_late1, "00001");
	strcpy(m_late2, "00001");
}

void seektest::cpu_execute()
{
	while (!get_quitflag())
	{
		SDL_check_input();
		SDL_Delay(10);	// don't hog CPU
	}
}

void go(char *s)
{
	g_ldp->pre_search(s);
}

void seektest::input_enable(Uint8 input)
{
	switch (input)
	{
	case SWITCH_UP:
		go(m_late1);
		break;
	case SWITCH_DOWN:
		go(m_late2);
		break;
	case SWITCH_LEFT:
		go(m_early1);
		break;
	case SWITCH_RIGHT:
		go(m_early2);
		break;
	case SWITCH_BUTTON1:
		g_ldp->pre_play();
		break;
	case SWITCH_BUTTON2:
		break;
	}
}

// set which disc you're testing
void seektest::set_preset(int val)
{
	switch (val)
	{
	case 0:	// dragon's lair
		disc_fps = 23.976;
		strcpy(m_early1, "00322");	// frame before attract mode
		strcpy(m_early2, "00323");	// 1st frame of attract mode
		strcpy(m_late1, "31615");	// black
		strcpy(m_late2, "31616");	// test pattern
		printline("Dragon's Lair disc test!");
		break;
	case 1:	// space ace
		disc_fps = 23.976;
		strcpy(m_early1, "01161");	// end of attract mode
		strcpy(m_early2, "01162");	// first frame of There's Borf's Ship
		strcpy(m_late1, "33185");	// black screen (end of victory)
		strcpy(m_late2, "33186");	// first frame of Dexter scene in final showdown
		printline("Space Ace disc test!");
		break;
	case 2:	// super don
		disc_fps = 29.97;
		strcpy(m_early1, "05400");	// black frame
		strcpy(m_early2, "05401");	// purple-blue gradiant frame 'super don quixote'
		strcpy(m_late1, "35850");	// black frame
		strcpy(m_late2, "35851");	// first frame of witch and the announcer saying 'game over'
		printline("Super Don disc test!");
		break;
	case 3:	// cliff hanger
		disc_fps = 29.97;
		strcpy(m_early1, "01544");	// last frame of attract mode with white text
		strcpy(m_early2, "01545");	// first frame of 1st stage still with white text
		strcpy(m_late1, "49665");	// blank screen
		strcpy(m_late2, "49666");	// first grey slide at end of disc
		break;
	case 4:	// astron belt
		disc_fps = 29.97;
									// there isn't a good early frame since astron
									// uses fades... one of the explosions would work
									// but all the explosions look the same :)
		strcpy(m_early1, "51330");	// black screen
		strcpy(m_early2, "51331");	// first frame of right speaker test
		break;
	case 5:	// galaxy ranger/star blazer
		disc_fps = 29.97;
		strcpy(m_early1, "1017");	// black screen
		strcpy(m_early2, "1018");	// start of launch scene right after titles
		strcpy(m_late1, "51667");	// black screen
		strcpy(m_late2, "51668");	// first frame of right speaker test
		break;
	case 6:	// Thayer's Quest
		disc_fps = 29.97;
		strcpy(m_early1, "00716");	// logo with light to the upper left of the T
		strcpy(m_early2, "00717");	// logo with light having moved to the h a little bit
		strcpy(m_late1, "49990");	// relics of quoid
		strcpy(m_late2, "49991");	// sorsabal telling the black magician to walk into the camera :)
		break;
	case 7:	// Cobra Command
		disc_fps = 29.97;
		strcpy(m_early1, "00300");	// black frame
		strcpy(m_early2, "00301");	// first frame that's not black.. helicopter coming over mountains
		strcpy(m_late1,  "46152");	// a totally white frame
		strcpy(m_late2,  "46153");	// a totally black frame
		break;
	case 8:	// Esh's Aurunmilla (info from Antonio "Ace64" Carulli)
		disc_fps = 29.97;
		strcpy(m_early1, "00450");	// black frame
		strcpy(m_early2, "00451");	// first frame of attract
		strcpy(m_late1, "39154");	// blue frame, last frame of last scene
		strcpy(m_late2, "39155");	// black frame
		break;
	case 9:	// Badlands
		disc_fps = 29.97;
		strcpy(m_early1, "00150");	// black frame
		strcpy(m_early2, "00151");	// first frame of attract
		strcpy(m_late1, "44249");	// last frame of color test
		strcpy(m_late2, "44250");	// first frame of cross hatch
		break;
	default:
		printline("SEEKTEST ERROR : unknown preset");
		break;
	}
}
