/*
 * conout.cpp
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// CONOUT
// Provides console functionality in a window (replacement for printf, cout, etc)

#include <stdio.h>
#include <string.h>
#include <SDL.h>
#include "conout.h"
#include "../daphne.h"
#include "../video/video.h"
#include "../video/SDL_Console.h"
#include "../timer/timer.h"
#include "input.h"

char G_curline[CHARS_PER_LINE] = { 0 };	// current line of output

// Prints a "c string" to the screen
// Returns 1 on success, 0 on failure
unsigned char outstr(char *s)
{
	unsigned char result = 1;

	if (get_console_initialized())
	{
		if (strlen(s) + strlen(G_curline) < CHARS_PER_LINE)
		{
			strcat(G_curline,s);
		}
	}
	addlog(s);	// add to our logfile

	return(result);
}

// Prints a single character to the screen
// Returns 1 on success, 0 on failure
unsigned char outchr(char ch)
{

	unsigned char result = 1;
	char s[2] = { 0 };

	s[0] = ch;
	s[1] = 0;

	outstr(s);

	return(result);

}


// prints a null-terminated string to the screen
// and adds a line feed
// 1 = success
unsigned char printline(char *s)
{

	unsigned char result = 1;

	outstr(s);
	newline();

	return(result);

}

// moves to the next line without printing anything
void newline()
{

	// Carriage return followed by line feed
	// So we can read the blasted thing in notepad
	static char newline_str[3] = { 13, 10, 0 };

	if (get_console_initialized())
	{
		ConOut(G_curline);
		G_curline[0] = 0;	// erase line
	}

	addlog(newline_str);

}

// prints the current line even if we haven't hit a linefeed yet
void con_flush()
{

	ConOutstr(G_curline);

}


// flood-safe printline
// it only prints every second and anything else is thrown out
void noflood_printline(char *s)
{
	static unsigned int old_timer = 0;
	
	if (elapsed_ms_time(old_timer) > 1000)
	{
		printline(s);
		old_timer = refresh_ms_time();
	}
}
