/*
 * error.cpp
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// error.cpp
// daphne error handling

#include <SDL.h>
#include <string.h>
#include "../daphne.h"
#include "conin.h"
#include "../sound/sound.h"
#include "../video/video.h"
#include "../video/SDL_DrawText.h"

const char *instr = "Please read the daphne_log.txt file for more information";

// notifies the user of an error that has occurred
void printerror(char *s)
{

	// if video has been initialized
	// FIXME: if bmp hasn't been loaded this might blow up
	if (get_console_initialized())
	{
		draw_othergfx_unsafe(B_DAPHNE_SAVEME, 0, 180);
		DrawText(s, get_screen(), FONT_SMALL, (400-((strlen(s)/2)*6)), 240-13);
		DrawText(instr, get_screen(), FONT_SMALL, (400-((strlen(instr)/2)*6)), 240);
		SDL_Flip(get_screen());
	}

	// play a 'save me' sound if we've got sound
	if (get_sound_initialized())
	{
		sound_play_saveme();
	}
#ifdef LINUX
	printf("%s\n",s);
#endif
	addlog(s);
	addlog("\n");
	con_getkey();	// wait for keypress

}

// notifies user that the game does not work correctly and gives a reason
// this should be called after video has successfully been initialized
void printnowookin(char *s)
{
	SDL_Rect region = { 0, 180, get_video_width(), (Uint16)(get_video_height()/2) };

	draw_othergfx_unsafe(B_GAMENOWOOK, 0, 180);
	DrawText(s, get_screen(), FONT_SMALL, (320-((strlen(s)/2)*6)), 240);
	SDL_Flip(get_screen());
	con_getkey();	// wait for keypress
	SDL_FillRect(get_screen(), &region, 0);	// erase message
	SDL_Flip(get_screen());
	SDL_Delay(2000);	// delay to make sure this doesn't interfere w/ overlay
}

// prints a notice to the screen
void printnotice(char *s)
{
	SDL_Rect region = { 0, 180, get_video_width(), (Uint16)(get_video_height()/2) };
	char ch = 0;

	DrawText(s, get_screen(), FONT_SMALL, (320-((strlen(s)/2)*6)), 240);
	SDL_Flip(get_screen());
	ch = con_getkey();	// wait for keypress
	SDL_FillRect(get_screen(), &region, 0);	// erase message
	SDL_Flip(get_screen());
	SDL_Delay(2000);	// delay to make sure this doesn't interfere w/ overlay
	
	// if they pressed escape, quit
	if (ch == 27)
	{
		set_quitflag();
	}
}