/*
 * ldp-vldp.h
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef LDP_VLDP_H
#define LDP_VLDP_H

#include <SDL.h>

struct fileframes1
{
	char name[81];	// name of mpeg file
	Sint32 frame;	// ldp frame that mpeg file starts on
};

#include "ldp.h"

#ifdef WIN32
#include <windows.h>	// for DLL loading
#else
#include <dlfcn.h>	// for .so loading
#endif

// macros for making the dynamic library code look cleaner
#ifdef WIN32
#define M_LOAD_LIB(libname)	LoadLibrary(#libname)
#define M_GET_SYM	GetProcAddress
#define M_FREE_LIB	FreeLibrary
#else
#define M_LOAD_LIB(libname)	dlopen("lib" #libname ".so", RTLD_NOW)
#define M_GET_SYM	dlsym
#define M_FREE_LIB	dlclose
#endif


class ldp_vldp : public ldp
{
public:
	ldp_vldp();

	bool init_player();
	void shutdown_player();
	bool search(char *);
	void play();
	void pause();
//	Uint16 get_current_frame();
	void enable_audio1();
	void enable_audio2();
	void disable_audio1();
	void disable_audio2();
	void request_screenshot();
	
	void set_framefile(char *filename);

private:
	bool load_vldp_lib();
	void free_vldp_lib();
	bool read_frame_conversions();
	bool first_video_file_exists();
	bool last_video_file_parsed();
	void parse_all_video();
	Uint16 mpeg_info (char *filename, Uint16 ld_frame);
	
	Sint32 m_target_mpegframe;	// mpeg frame # we are seeking to
	Sint32 m_cur_ldframe_offset;	// which laserdisc frame corresponds to the first frame in current mpeg file
	char m_cur_mpeg_filename[160];	// name of the mpeg file we currently have open
	char m_mpeg_path[160]; // location of mpeg file(s) to play
	char m_framefile[160];	// name of the framefile we load to get the names of the mpeg files and their frame #'s
	struct fileframes1 m_mpeginfo[300]; // names of mpeg files
	int m_file_index; // # of mpeg files in our list
#ifdef WIN32
	HINSTANCE m_dll_instance;	// pointer to DLL we load
#else
	void *m_dll_instance;	// pointer to .so we load
#endif

	bool m_audio_file_opened;	// whether we have audio to accompany the video

	// stuff inside ldp-vldp-audio.cpp
	void oggize_path(char *, char *);
	bool audio_init();
	void audio_shutdown();
	void close_audio_stream();
	bool open_audio_stream(char *filename);
	bool seek_audio(double time);
	void audio_play();
	void audio_pause();

	

};

void yuv_callback(SDL_Overlay *dst);

#endif
