/*
 * ldp.h
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#ifndef LDP_PRE_H
#define LDP_PRE_H

#include <SDL.h>	// needed for datatypes

#define FRAME_SIZE 5	

class ldp
{
public:
	ldp();
	bool init();
	virtual bool init_player();
	void shutdown();
	virtual void shutdown_player();
	void clear();	// clears any received frames
	bool pre_search(char *);
	virtual bool search(char *);
	bool pre_skip_forward(Uint16);
	virtual bool skip_forward(Uint16);
	void pre_play();
	virtual void play();
	virtual void set_play_speed(Uint8);	// sets the speed we playback at
	void pre_pause();
	virtual void pause();
	void pre_stop();
	virtual void stop();
	virtual Uint16 get_current_frame();
	void pre_step_forward();
	virtual void enable_audio1();
	virtual void enable_audio2();
	virtual void disable_audio1();
	virtual void disable_audio2();
	virtual void request_screenshot();

	bool is_blitting_allowed();	// returns value of blitting_allowed
	bool is_disc_playing();	// returns value of is_playing
	void framenum_to_frame(Uint16, char *);	// converts int to 5-digit string
	Uint32 get_search_latency();
	void set_search_latency(Uint32);

protected:
	bool need_serial;	// whether this LDP driver needs the serial port initialized
	bool serial_initialized; // whether serial has been initialized
	bool player_initialized; // whether the LDP has been properly initialized
	bool blitting_allowed;	// whether it's ok to blit directly to the screen (SMPEG forbids this)
	bool skipping_supported;	// whether the laserdisc player supports skipping
	bool skip_instead_of_search;	// whether we should skip instead of search if searching forward a short distance
	Uint16 max_skippable_frames;	// maximum # of frames that player can skip (if skipping is supported)
	Uint16 last_frame;	// the last frame we seeked to
	unsigned int play_time;	// current time when we last played
	bool is_playing;	// whether disc is currently playing
	Uint32 search_latency;	// how many ms to stall before searching (to simulate slow laserdisc players)
	bool stop_on_quit;	// should the LDP stop when it quits?

	Uint32 test_timer;	// remove me
	bool started_search;	// remove me
};

#endif
