/*
 * rgb2yuv.cpp
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


// RGB2YUV.CPP
// by Matt Ownby

#include "rgb2yuv.h"

// converts RGB color components to YUV
// RGB values should range from 0-255
yuvcolors rgb2yuv(unsigned char R, unsigned char G, unsigned char B)
{
	yuvcolors result;

	result.y = ((double) R *  .299) + ((double) G *  .587) +
			((double)  B *  .114);
	result.u = ((double) R * -.169) + ((double) G * -.332) +
			((double)  B *  .500) + 128;
	result.v = ((double) R *  .500) + ((double) G * -.419) +
			((double)  B * -.0813) + 128;
	return(result);

}

// a faster, less accurate version of the RGB->YUV conversion formula
// (faster because no floating point is used)
yuvcolors8 rgb2yuv8(unsigned char R, unsigned char G, unsigned char B)
{
	yuvcolors8 result;

	result.y = (unsigned char) (((9798*R)+(19235*G)+(3736*B))/32768);
	result.u = (unsigned char) ((((-5537*R)+(-10878*G)+(16384*B))/32768) + 128);
	result.v = (unsigned char) ((((16384*R)+(-13729*G)+(-2664*B))/32768) + 128);

	return(result);

}

// a different algorithm to convert RGB to YUV from intel
yuvcolors intelrgb2yuv(unsigned char R, unsigned char G, unsigned char B)
{
	yuvcolors result;

	result.y = ((double) R *  .299) + ((double) G *  .587) +
			((double)  B *  .114);
	result.u = ((double) R * -.146) + ((double) G * -.288) +
			((double)  B *  .434);
	result.v = ((double) R *  .617) + ((double) G * -.517) +
			((double)  B * -.100);
	return(result);

}
