// ldp-smpeg.c
// by Matt Ownby

// This file is still pretty rough, lots of hacks

#include "yourframeinfo.h"	// make this file yourself =]

// pretends to be an LDP, but uses the smpeg library for output
// (for people who have no laserdisc player)

#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#ifdef LINUX
#include <unistd.h>	// for fstat and unlink
#endif
#include <time.h>
#include "../io/conout.h"
#include "../video/video.h"
#include "../io/error.h"
#include "ldp-smpeg.h"
#include "../timer/timer.h"
#include "../daphne.h"
#include "../io/input.h"
#include <SDL_mixer.h>
#include "../sound/sound.h"
#include "../game/game.h"
#include "rgb2yuv.h"
#include "mpegscan.h"

extern game *g_game;	// global game class (needed to query disc FPS)

bool g_left_channel_muted = false;	// it's easier to have these global for easy access and speed
bool g_right_channel_muted = false;	

bool g_take_screenshot = false;	// if true, a screenshot will be taken at the next available opportunity

bool g_display_video = true;	// if this is false, our video callback will erase the frame

/////////////////////////////////////////

ldp_smpeg::ldp_smpeg()
{
	blitting_allowed = false;	// SMPEG freaks out if we blit to the screen, not sure why
	m_smpeg_paused = false;
	m_target_mpegframe = 0;	// mpeg frame # we are seeking to
	m_mpeg = NULL; // current mpeg that is loaded
	m_info_ptr = NULL;
	memset( (void *) &m_mpeg_path[0], 0, 160);
	m_file_index = 0; // # of mpeg files in our list
}

// called when daphne starts up
bool ldp_smpeg::init_player()
{
	bool result = false;
	
	result = read_frame_conversions();

	return result;
}

void ldp_smpeg::shutdown_player()
{
	// if we've got an mpeg loaded, de-allocate it
	if (m_mpeg)
	{
		SMPEG_stop(m_mpeg);
		SMPEG_delete(m_mpeg);
		Mix_HookMusic(NULL, NULL); // unhook SMPEG from sdl mixer
	}
	m_mpeg = NULL;
	
}

bool ldp_smpeg::search(char *frame)
{
	
	bool result = false;
	char filename[81] = { 0 };
//	SDL_AudioSpec audiofmt;
//	Uint16 format;
	int freq = 0;
	int channels = 0;
	Uint16 target_ld_frame = (Uint16) atoi(frame);
	
	// FIXME: determine if we are seeking inside a file
	// we already have open

	// FIXME: For now we assume we are only using 1 file
	// this of course won't work for Dragon's Lair Digital Leisure

	m_target_mpegframe = mpeg_info(filename, target_ld_frame); // try to get a filename
	
	// if the mpeg's FPS and the disc's FPS differ, we need to adjust the mpeg frame
	// NOTE: AVOID this if you can because it makes seeking less accurate
	if (g_game->get_disc_fps() != MPEGFPS)
	{
		printf("NOTE: converting FPS, you should really fix this :)\n");
		m_target_mpegframe = (Uint16) (((m_target_mpegframe / g_game->get_disc_fps()) * MPEGFPS) + 0.5); // add 0.5 to round to nearest whole number when truncating
	}

	// if we can convert target frame into a filename, do it!
	if (filename[0] != 0)
	{
		// if we're open the file for the first time
		if (!m_mpeg)
		{
			m_mpeg = SMPEG_new(filename, &m_info, 0);
			// NOTE: we're supposed to have this set to 0
			// in order to use sdl mixer..
			
			// if mpeg did not load correctly
			if (SMPEG_error (m_mpeg))
			{
				printerror(SMPEG_error(m_mpeg));
				SMPEG_delete(m_mpeg);
				m_mpeg = NULL;
			}
			else
			{
				SMPEG_enableaudio(m_mpeg, 0);
				SMPEG_setdisplay(m_mpeg, get_screen(), NULL, NULL);
				SMPEG_enablevideo(m_mpeg, 1); // turn on video

				// set up video overlay filter
				SMPEG_Filter *filter;
				filter = SMPEGfilter_vo();	// my new one
				filter = SMPEG_filter( m_mpeg, filter );
				filter->destroy(filter);

				// hook into our customized sound callback
// MATT: I disabled audio since when seeking it's not in sync w/ the video and it's driving me
// crazy hehe
/*
				Mix_QuerySpec(&freq, &format, &channels);
				audiofmt.format = format;
				audiofmt.freq = freq;
				audiofmt.channels = (unsigned char) channels;
				SMPEG_actualSpec(m_mpeg, &audiofmt);
				Mix_HookMusic(ldp_smpeg_mix, m_mpeg); // our own mix callback to mute channels
				SMPEG_enableaudio(m_mpeg, 1);
*/
				result = true;
			}
		}
		// if we're playing video, stop while we "search"
		else if (is_playing)
		{
			pause();
			result = true;
		}
		
		// else we are paused when we do the search
		else
		{
			result = true;
		}
	}
	
	else
	{
		printline("smpeg search: no filename returned, ignoring");
	}

	if (result)
	{
		m_need_search = true;
	}
	
	return(result);
}

void ldp_smpeg::play()
{
	char s[81] = { 0 };
	time_t blah = time(NULL);
	Uint32 pos = 0;
	int old_frame = 0;	// frame we are on before we seek

	// if we have an mpeg file loaded
	if (m_mpeg)
	{

		g_display_video = false;	// hide video temporarily until we start playing

		// if we are not at a still, frame then play from the beginning of the file
		if (!m_smpeg_paused)
		{
			SMPEG_play(m_mpeg);
			printline("Sending play command...");
		}
		else
		{
			SMPEG_pause(m_mpeg);	// this unpauses
			printline("Unpausing SMPEG...");
			m_smpeg_paused = false;
		}

		// only search if one has been requested
		if (m_need_search)
		{		
			SMPEG_getinfo(m_mpeg, &m_info);
			old_frame = m_info.current_frame;

			m_target_mpegframe -= 2;	// adjust for smpeg peculiarities

			if (m_target_mpegframe < (Sint32) m_totalframes)
			{
				if (m_target_mpegframe >= 0)
				{
				  pos = m_frame_position[m_target_mpegframe];
				  printf("target frame is %u\n", m_target_mpegframe);
				  printf("the target frame pos %u\n", m_frame_position[m_target_mpegframe]);
				}
			}
			else
			{
				printline("SMPEG error: requested mpeg frame is out of range");
				printf("m_totalframes is %d\n", m_totalframes);
			}

			SMPEG_seek(m_mpeg, (int) (pos));

			sprintf(s, "Seeking to position %d", (int) pos);
			printline(s);
			m_need_search = false;
		}

		while (!get_quitflag())
		{
			SDL_check_input(); // see if we need to abort
			SMPEG_getinfo(m_mpeg, &m_info);

			if (blah != time(NULL))
			{
				sprintf(s,"Current frame is %d, target frame is %d\n", m_info.current_frame, m_target_mpegframe);
				printline(s);
				blah = time(NULL);
			}
			
			if (m_info.current_frame != old_frame)
			{
				printline("SMPEG: We seem to be playing now!");
				g_display_video = true;	// turn on video once we are moving
				break;

			}
			SDL_Delay(10);
		}		
	}
	else
	{
		printline("Tried to play without having a frame");
	}
}

void ldp_smpeg::pause()
{
	if (m_mpeg)
	{
		printline("SMPEG: Pausing");
		SMPEG_pause(m_mpeg);
		m_smpeg_paused = true;
	}
	else
	{
		printline("SMPEG: Pause command received but no file is loaded");
	}
}

// enables left audio channel
void ldp_smpeg::enable_audio1()
{
	printline("*** audio 1 enabled");
	g_left_channel_muted = false;
}

// enables right audio channel
void ldp_smpeg::enable_audio2()
{
	printline("*** audio 2 enabled");
	g_right_channel_muted = false;
}

// disables left audio channel
void ldp_smpeg::disable_audio1()
{
	g_left_channel_muted = true;
}

// disables right audio channel
void ldp_smpeg::disable_audio2()
{
	g_right_channel_muted = true;
}

// takes a screenshot of the current smpeg frame + any video overlay
void ldp_smpeg::request_screenshot()
{
	g_take_screenshot = true;
}

// read frame conversions in from LD-frame to mpeg1-frame data file
bool ldp_smpeg::read_frame_conversions()
{

	FILE *fileConvert = NULL;
	char s[81] = { 0 };
	int cAssigned;
	char frame_string[81] = { 0 };
	Uint32 frame;
	bool result = false;

	fileConvert = fopen(FRAMEFILE, "r");

	if (fileConvert)
	{
		fscanf(fileConvert, "%s", m_mpeg_path); // read in path

		// read entire file
		while(!feof(fileConvert))
		{

			cAssigned = fscanf(fileConvert, "%s %s", frame_string, s);
			frame = atoi(frame_string);
				               
			// if we read 2 entries like we expected, store them
			if(cAssigned == 2)
			{
				m_mpeginfo[m_file_index].frame = (Uint16) frame;
				strcpy(m_mpeginfo[m_file_index].name, s);
				m_file_index = m_file_index + 1;
			}
		}

		result = 1;	
		fclose(fileConvert);
	}
	else
	{
		printerror("Could not open mpeg frame conversion file");
	}
	
	return result;
}

// get the position of all the frames in the mpeg
// returns true on success or false on failure
bool ldp_smpeg::get_frame_positions(char *mpeg_name)
{

	bool result = true;
	bool mpeg_datafile_good = false;
	char datafilename[81] = { 0 };
	FILE *mpeg_file = NULL, *data_file = NULL;
	Uint32 alleged_mpeg_size = 0;	// how big the datafile says the mpeg is
	char full_path[160] = { 0 };	// full path of the mpeg file
	Uint32 mpeg_size = 0;	// size of the mpeg file
	struct stat filestats;	// used to get the size of a file

	strcpy(full_path, m_mpeg_path);
	strcat(full_path, mpeg_name);	// create the full path name

	mpeg_file = fopen(full_path, "rb");	// try to open the mpeg file

	// if we could open the file successfully, get its length
	if (mpeg_file)
	{
		fstat(fileno(mpeg_file), &filestats);	// get stats for file to get file length
		mpeg_size = filestats.st_size;
	}
	else
	{
		outstr("ERROR: could not open mpeg file ");
		printline(full_path);
		printline("DAPHNE will be shut down until you can fix this problem");
		printerror("Could not open mpeg file!");
		result = false;
		set_quitflag();
	}

	// change extension of file to be dat instead of (presumably) mpg
	strcpy(datafilename, mpeg_name);
	strcpy(&datafilename[strlen(mpeg_name)-3], "dat");

	// loop until we get a good datafile or until user quits
	while (!mpeg_datafile_good && !get_quitflag())
	{
		// keep trying to make the datafile until we succeed of the user quits
		while (!data_file && !get_quitflag())
		{
			SDL_check_input();
			data_file = fopen(datafilename, "rb");	// check to see if datafile exists

			// if file does not exist, we have to create it
			if (!data_file)
			{
				data_file = fopen(datafilename, "wb");	// create file

				// if we could create the file successfully, then we need to populate it
				if (data_file)
				{
					Uint32 pos = 0;	// position in the file
					Sint32 status = 0;	// status of parse
					char s[81] = { 0 };
					double percentage_completed = 0.0f;
					int count = 0;

					printline("MPEG frame data does not exist and must be generated.");
					printline("Please be patient, this could take a few minutes...");

					fwrite(&mpeg_size, sizeof(mpeg_size), 1, data_file);
					// first thing that goes in the file is the length of the mpeg
					// That way we can re-use the file another time with confidence that it's
					// the right one

					// keep reading the file while there is a file left to be read
					do
					{
						pos = parse_system_stream(mpeg_file, data_file, status);

						// we want to give the user updates but don't want to flood them
						if (count > 1000)
						{
							percentage_completed = (pos * 100.0) / mpeg_size;
							sprintf(s, "MPEG file parsing is %f percent complete", percentage_completed);
							printline(s);
							printf("%u and %d\n", pos, mpeg_size);
							count = 0;
						}
						count++;
						SDL_check_input();
					} while ((status == P_IN_PROGRESS) && (!get_quitflag()));

					// if the mpeg did not finish parsing gracefully, we've got problems
					if (status != P_FINISHED)
					{
						printline("There was an error parsing the MPEG file.");
						printline("Either there is a bug in the parser or the MPEG file is corrupt.");
						set_quitflag();
						result = false;
					}

					fclose(data_file);
					data_file = NULL;
					// we have to close data file because it's write-only
					// and it needs to be re-opened read-only
					
				} // end if we could create the file successfully

				// we couldn't create data file which means no write permission probably
				// this is probably a good time to shut daphne down =]
				else
				{
					outstr("Could not create file ");
					printline(datafilename);
					printline("This probably means you don't have permission to create the file");
					printline("SMPEG can't operate without this file, so we're gonna shut daphne");
					printline("down until you fix the problem.");
					printerror("Could not create mpeg data file!");
					set_quitflag();
					result = false;
				}
			} // end if the data file did not exist and we had to create
		} // end of inner while that tries to open the datafile

		// now that file exists and we have it open, we have to read it

		fseek(data_file, 0L, SEEK_SET);
		int bytes_read = fread(&alleged_mpeg_size, 4, 1, data_file);	// read size mpeg is supposed to be

		// if file size is wrong, the mpeg datafile does not apply and thus we need to redo it
		if (alleged_mpeg_size != mpeg_size)
		{
			printf("*** Alleged mpeg size is %u, actual size is %u\n", alleged_mpeg_size,
				mpeg_size);
			printline("MPEG size does not match, MPEG data file has to be created again!");
			fclose(data_file);
			data_file = NULL;
			unlink(datafilename);	// delete file and create it
		}
		else
		{
			mpeg_datafile_good = true;
		}
	} // end while we don't have a good datafile

	// if we didn't exit the loop because someone quit, then we need to read the datafile
	// data_file is still open
	if (!get_quitflag() && data_file)
	{
		m_totalframes = 0;

		// read a frame position
		// if we don't read 4 bytes, it means we've hit the EOF and we're done
		while (fread(&m_frame_position[m_totalframes], 4, 1, data_file) == 1)
		{
			m_totalframes++;
		}
		printf("*** m_totalframes is %u\n", m_totalframes);
	}

	if (data_file)
	{
		fclose(data_file);
	}
	if (mpeg_file)
	{
		fclose(mpeg_file);
	}

	return(result);

}

// returns # of frames to seek into file, and mpeg filename
// if there's an error, filename is NULL
// WARNING: This assumes the mpeg and disc are running at exactly the same FPS
// If they aren't, you will need to calculate the actual mpeg frame to seek to
// The reason I don't return time here instead of frames is because it is more accurate to
//  return frames if they are at the same FPS (which hopefully they are hehe)
Uint16 ldp_smpeg::mpeg_info (char *filename, Uint16 ld_frame)
{

	static char old_filename[81] = { 0 };	// used to determine if we are inside the same file or not

	int index = 0;
	Uint16 mpeg_frame = 0;	// which mpeg frame to seek (assuming mpeg and disc have same FPS)

	// find the mpeg file that has the LD frame inside of it
	while ((index+1 < m_file_index) && (ld_frame >= m_mpeginfo[index+1].frame))
	{
		index = index + 1;
	}

	// make sure we have a filename at this spot
	if (m_mpeginfo[index].name[0])
	{
		strcpy(filename, m_mpeg_path);
		strcat(filename, m_mpeginfo[index].name);
		mpeg_frame = (Uint16) (ld_frame - m_mpeginfo[index].frame);

		// if the mpeg we will use is different from the one we used before, we have to load
		// the mpeg data
		if (strcmp(m_mpeginfo[index].name, old_filename) != 0)
		{
			get_frame_positions(m_mpeginfo[index].name);
			strcpy(old_filename, m_mpeginfo[index].name);
		}
	}
	else
	{
		printline("SMPEG error, no filename found");
		filename[0] = 0;	// ERROR
	}
	
	return(mpeg_frame);
}

////////////////////////////////////////////////////////////////////////////////////////



/* General filter cleanup function */
void filter_destroy(SMPEG_Filter *filter)
{
    if ( filter ) {
        if ( filter->data ) {
            free(filter->data);
        }
        free(filter);
    }
}

// Custom filter callback I wrote so that we can draw lasergame video
// overlay on top of mpeg video
void filter_vo_callback(SDL_Overlay * dst, SDL_Overlay * src, SDL_Rect * region, SMPEG_FilterInfo * info, void * data)
{

	// if video ought to be displayed then run it through a bilinear filter
	if (g_display_video)
	{
		// now run the whole image through the bilinear filter to smooth out artifacts
		SMPEGfilter_bilinear()->callback(dst, src, region, info, data);
	}
	// else video should be blanked which we accomplish with the following code
	else
	{
		Uint8 *Y = dst->pixels[0];
		Uint8 *V = dst->pixels[1];
		Uint8 *U = dst->pixels[2];
		int count;
		for (count = 0; count < (region->w * region->h); count++)
		{
			*(Y++) = 0;
		}
		for (count = 0; count < ((region->w * region->h) >> 2); count++)
		{
			*(V++) = 127;
			*(U++) = 127;
		}
	}

	// now adjusted image has been displayed
	// now we put the video overlay on top of it  

	Uint8 transparent_color = 0;
	SDL_Surface *gamevid = g_game->get_video_overlay(transparent_color);
	SDL_Color *gamecolors = g_game->get_video_colors();

	// if the game we're playing has a video overlay
	if (gamevid && gamecolors)
	{
		int vertical_padding = g_game->get_video_row_offset();
		Uint8 *Y = dst->pixels[0] + (dst->pitches[0] * vertical_padding);	// the Y channel
		Uint8 *V = dst->pixels[1] + ((region->w >> 2) * vertical_padding);	// V channel
		Uint8 *U = dst->pixels[2] + ((region->w >> 2) * vertical_padding);	// U channel
		struct yuvcolors8 yuv_components = { 0, 0, 0 };	// the YUV components of the game video pixels
		int row = 0;
		int col = 0;

		Uint8 *gamevid_pixels = (Uint8 *) gamevid->pixels;
		Uint8 *temp = NULL;	// temporary pointer
		Uint8 cur_color = 0;
		int last_color = -1;	// the last color we converted to YUV
		// made an integer so I could set it to -1 and not risk overlapping
		// any legitimate 8-bit color

		// go through each row of the overlay and copy it to the YUV stream
		for (row = 0; (row < gamevid->h) && (row < dst->h); row ++)
		{
			for (col = 0; col < gamevid->w; col++)
			{
				cur_color = *gamevid_pixels;
				
				// if current game video overlay pixel is not transparent
				// then write it to the YUV stream
				if (cur_color != transparent_color)
				{
					// RGB2YUV conversion is expensive so we only want to do it if it
					// necessary
					if (cur_color != last_color)
					{
						// convert RGB to YUV since our mpeg stream is in YUV format
						yuv_components = rgb2yuv8(gamecolors[cur_color].r,
							gamecolors[cur_color].g,
							gamecolors[cur_color].b);
						last_color = cur_color;
					}
						
					// we double size of pixel so we need to write
					// the value in 4 places
					
					// this plots a 2x2 square (a 1x1 pixel doubled in size)
					*Y = yuv_components.y;
					*(Y+1) = yuv_components.y;
					temp = Y + region->w;
					*temp = yuv_components.y;
					*(temp+1) = yuv_components.y;

					// U and V are half the size of Y so we do not need to enlarge
					*V = yuv_components.v;
					*U = yuv_components.u;
				}
				gamevid_pixels++;	// advance to next pixel
				Y += 2;	// advance two pixels over since game video is doubled
				U++;
				V++;
			}
			Y += region->w;
		}
	} // end if overlay exists

	// if we've been instructed to take a screenshot
	if (g_take_screenshot)
	{
		g_take_screenshot = false;
		take_screenshot(dst);
	}
}


// sets up smpeg to use specialized filter which applies video overlay from some games
SMPEG_Filter *SMPEGfilter_vo(void)
{
    SMPEG_Filter *filter;

	printf("My video overlay filter being chosen...\n");

    filter = (SMPEG_Filter *)malloc(sizeof(*filter));
    if ( filter ) {
        /* the null filter requires no extra info */
        filter->flags = 0;
        /* no private data */
        filter->data = 0;
        /* set the function pointers to the callback and destroy functions */
        filter->callback = filter_vo_callback;
        filter->destroy = filter_destroy;
    }
    return filter;
}

// serves as an intermediate function between the smpeg mix
// and sdl mixer so we can mute channels if we need to
// (neither smpeg nor sdl mixer allow you to mute channels)
void ldp_smpeg_mix(void *mpeg, Uint8 *stream, int len)
{
	int index = 0;
	
	SMPEG_playAudioSDL(mpeg, stream, len);	// let SMPEG do its mix
	
	// make sure we can work with the audio stream correctly	
	if (len >= 4 && (len % 2 == 0))
	{
		Uint8 *ptr = stream;
		Uint8 soundlo = 0, soundhi = 0;

		// test to mute sound of the channels
		for (index = 0; index < len-4; index+=4)
		{
			if (g_left_channel_muted)
			{
				// if they're both muted, kill sound
				if (g_right_channel_muted)
				{
					// fill stream with 0's
					for (int i = 0; i < 4; i++)
					{
						*ptr = 0;
						ptr++;
					}
				}
				// only left channel is muted, so copy right
				// channel sound to left channel
				else
				{
					Uint8 *ptr2 = ptr;
					ptr2++;
					ptr2++;
					soundhi = *ptr2;
					ptr2++;
					soundlo = *ptr2;
					ptr2++;
					
					*ptr = soundhi;
					ptr++;
					*ptr = soundlo;
					ptr++;
					
					ptr++;
					ptr++;
				}
			}
			// if right channel is muted but left is not,
			// then copy left channel to right channel
			else if (g_right_channel_muted)
			{
				soundhi = *ptr;
				ptr++;
				soundlo = *ptr;
				ptr++;
				*ptr = soundhi;
				ptr++;
				*ptr = soundlo;
				ptr++;
			}
			// else neither channel is muted, do nothing
		}
	//	printf("Out ok\n");
	}
	else
	{
		printline("SMPEG audio error: unexpected stream size");
	}
}
