#ifndef LDP_SMPEG_H
#define LDP_SMPEG_H

#define MAX_LDP_FRAMES 53946
// the maximum # of possible frames one side of a 30-minute CAV NTSC disc could have
// (30 minutes * (60 seconds/min) * (29.97 frames/sec))

struct fileframes
{
	char name[81];	// name of mpeg file
	Uint16 frame;	// ldp frame that mpeg file starts on
};

#include <SDL.h>
#include <smpeg.h>
#include "../ldp-out/ldp.h"

class ldp_smpeg : public ldp
{
public:
	ldp_smpeg();
	bool init_player();
	void shutdown_player();
	bool search(char *);
	void play();
	void pause();
	void enable_audio1();
	void enable_audio2();
	void disable_audio1();
	void disable_audio2();
	void request_screenshot();

private:
	bool read_frame_conversions();
	bool ldp_smpeg::get_frame_positions(char *mpeg_name);
	Uint16 mpeg_info (char *filename, Uint16 ld_frame);

	bool m_smpeg_paused;
	bool m_need_search;	// if we need to search when we play
	Sint32 m_target_mpegframe;	// mpeg frame # we are seeking to
	SMPEG *m_mpeg; // current mpeg that is loaded
	SMPEG_Info m_info;
	SMPEG_Info *m_info_ptr;
	char m_mpeg_path[160]; // location of mpeg1 files to play
	struct fileframes m_mpeginfo[300]; // names of mpeg files
	int m_file_index; // # of mpeg files in our list
	unsigned int m_frame_position[MAX_LDP_FRAMES];	// the file position of each frame
	Uint16 m_totalframes;	// total frames in the current mpeg file


};

void filter_destroy(SMPEG_Filter *filter);
void filter_vo_callback(SDL_Overlay * dst, SDL_Overlay * src, SDL_Rect * region, SMPEG_FilterInfo * info, void * data);
SMPEG_Filter *SMPEGfilter_vo(void);
void ldp_smpeg_mix(void *mpeg, Uint8 *stream, int len);

#endif