/*
 * timer.cpp
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// by Matt Ownby, 11 August 1999

// Purpose: To provide a way for our emulator to track very precise time
// Second purpose: To keep this source code portable


#include <time.h>
#include <sys/timeb.h>
#include <SDL.h>

#ifdef WIN32
#include <windows.h>
#endif

#ifdef LINUX
#include <unistd.h>
#endif


// returns a 32-bit millisecond timer value
unsigned int refresh_ms_time()
{

	unsigned int result = 0;

#ifdef LINUX

	static struct timeb t;

	ftime(&t);
	result = t.time;
	result = result * 1000;	// convert seconds into milliseconds
	result = result + t.millitm;	// add the milliseconds on the end
#endif

#ifdef WIN32

	result = timeGetTime();
	// NOTE: This requires linking with winmm.lib

#endif

	return(result);

}

// returns the elapsed time (in milliseconds) since the current
// time and previous_time (which is also in milliseconds)
unsigned int elapsed_ms_time(unsigned int previous_time)
{

	return(refresh_ms_time() - previous_time);

}





// Puts the program to sleep for 'howlong' milliseconds
void make_delay(int howlong)
{

	SDL_Delay(howlong);

/*
#ifdef WIN32

	Sleep(howlong);	// sleep for this many milliseconds

#endif

#ifdef LINUX

	usleep(howlong * 1000);
	// usleep is in microseconds, so we multiply by 1000 to convert to milliseconds

#endif
*/

}
