/*
 * led.cpp
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of DAPHNE, a laserdisc arcade game emulator
 *
 * DAPHNE is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * DAPHNE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//LED.CPP - By Brad Oldham for Space Ace skill LED emulation

#include "led.h"

#ifdef WIN32
#include <windows.h>
#endif

bool g_save_numlock=false, g_save_capital=false, g_save_scroll=false;
bool g_leds_enabled = false;	// LEDs are disabled by default since they don't work on all platforms

// changes keyboard LED lights to simulate the skill LEDs in Space Ace
// changed_led (false, false, false) set all 3 lights to off
// changed_led (true, true, true) sets all 3 lights to on.

void change_led(bool num_lock, bool caps_lock, bool scroll_lock)
{

	// are the LED's enabled
	if (g_leds_enabled)
	{
#ifdef WIN32
		
		if ((num_lock==true && GetKeyState(VK_NUMLOCK) == 0x00) ||
			(num_lock==false && GetKeyState(VK_NUMLOCK) == 0x01))
			{
			keybd_event(VK_NUMLOCK, 0x45, KEYEVENTF_EXTENDEDKEY | 0, 0);
			keybd_event(VK_NUMLOCK, 0x45, KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, 0);
			}

		if ((caps_lock==true && GetKeyState(VK_CAPITAL) == 0x00) ||
			(caps_lock==false && GetKeyState(VK_CAPITAL) == 0x01))
			{
			keybd_event(VK_CAPITAL, 0x3a, 0, 0);
			keybd_event(VK_CAPITAL, 0x3a, KEYEVENTF_KEYUP, 0);
			}	

		if ((scroll_lock==true && GetKeyState(VK_SCROLL) == 0x00) ||
			(scroll_lock==false && GetKeyState(VK_SCROLL) == 0x01))	
			{
			keybd_event(VK_SCROLL, 0x46, 0, 0);
			keybd_event(VK_SCROLL, 0x46, KEYEVENTF_KEYUP, 0);
			}
#endif
	} // end whether LED's are enabled
}

void remember_leds()
{

	if (g_leds_enabled)
	{
#ifdef WIN32		

		if (GetKeyState(VK_NUMLOCK) == 0x00) 
			g_save_numlock = false;
		else
			g_save_numlock = true;

		if (GetKeyState(VK_CAPITAL) == 0x00)
			g_save_capital = false;
		else
			g_save_capital = true;

		if (GetKeyState(VK_SCROLL) == 0x00)
			g_save_scroll = false;
		else
			g_save_scroll = true;			
#endif
	}
}

void restore_leds() 
{
	change_led(g_save_numlock, g_save_capital, g_save_scroll);
}

// enables the LEDs for use since they are disabled by default
void enable_leds()
{
	g_leds_enabled = true;
}