/*
 * vldp_common.h
 *
 * Copyright (C) 2001 Matt Ownby
 *
 * This file is part of VLDP, a virtual laserdisc player.
 *
 * VLDP is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * VLDP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// shared by the public and private VLDP threads
// nothing else should include this

#include <SDL.h>
#include <SDL_thread.h>

extern SDL_mutex *g_vldp_mutex;

extern Uint16 g_req_frame;
extern Uint16 g_ack_frame;
extern int g_req_quit;
extern int g_ack_quit;
extern int g_req_play;
extern int g_ack_play;
extern int g_req_stop;
extern int g_ack_stop;
extern int g_req_shutdown;
extern int g_req_open;
extern int g_ack_open;
extern char g_req_file[];
extern int g_req_search;
extern int g_ack_search;
extern int g_req_pause;
extern int g_ack_pause;
extern int g_req_step_forward;
extern int g_ack_step_forward;
extern int g_ack_status;	// the current status of the VLDP (see enums)
extern double g_fps;	// framerate of the mpeg
extern Uint8 g_uses_fields;	// whether mpeg uses fields or frames
extern void (*g_yuv_callback)(SDL_Overlay *cur_overlay);

int idle_handler(void *);

// how ms to wait for responses from the private thread before we give up and return an error
#define VLDP_TIMEOUT	5000

// macros to lock and unlock these variables from being read/written
#define VLDP_LOCK	SDL_mutexP(g_vldp_mutex)
#define VLDP_UNLOCK	SDL_mutexV(g_vldp_mutex)

// makes a request to the private thread
#define VLDP_REQUEST(var)	\
	VLDP_LOCK;	\
	g_ack_ ## var = 0;	/* make sure the ack is clear */ \
	g_req_ ## var = 1;	\
	VLDP_UNLOCK;

// acknowledges a request to the public thread
#define VLDP_ACK(var)	\
	VLDP_LOCK;	\
	g_ack_ ## var = 1;	\
	g_req_ ## var = 0;	\
	VLDP_UNLOCK;

// waits for an acknowledgement from the other thread
#define VLDP_WAIT_FOR_ACK(var)	\
{	\
	int got_ack = 0;	\
	/* wait until we get an ACK from thread */	\
	while (!got_ack && !g_req_quit)	\
	{	\
		SDL_Delay(1);	\
		VLDP_LOCK;	\
		/* if we get an ack, clear both req and ack */	\
		if (g_ack_ ## var)	\
		{	\
			got_ack = 1;	\
			g_ack_ ## var = 0;	\
			g_req_ ## var = 0;	\
		}	\
		VLDP_UNLOCK;	\
	}	\
}
